<?php

namespace Drupal\pluggable_entity_view_builder_example\Plugin\EntityViewBuilder;

use Drupal\Core\Url;
use Drupal\node\NodeInterface;
use Drupal\pluggable_entity_view_builder\ComponentWrapTrait;
use Drupal\pluggable_entity_view_builder\EntityViewBuilderPluginAbstract;
use Drupal\pluggable_entity_view_builder_example\ProcessedTextBuilderTrait;
use Drupal\pluggable_entity_view_builder_example\TagBuilderTrait;

/**
 * The "Node Article" plugin.
 *
 * @EntityViewBuilder(
 *   id = "node.article",
 *   label = @Translation("Node - Article"),
 *   description = "Node view builder for Article bundle."
 * )
 */
class NodeArticle extends EntityViewBuilderPluginAbstract {

  use ComponentWrapTrait;
  use ProcessedTextBuilderTrait;
  use TagBuilderTrait;

  /**
   * The image style to use on Hero images.
   */
  const IMAGE_STYLE_LARGE = 'large';

  /**
   * Build full view mode.
   *
   * @param array $build
   *   The existing build.
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   *
   * @return array
   *   Render array.
   */
  public function buildFull(array $build, NodeInterface $entity) {
    $build['header'] = $this->buildHeroHeader($entity);
    $build['tags'] = $this->buildContentTags($entity);
    $build['body'] = $this->buildBody($entity);

    $build['comments'] = $this->buildComment($entity);

    // Load Tailwind CSS framework, so our example are styled nicer.
    $build['#attached']['library'][] = 'pluggable_entity_view_builder_example/tailwind';

    return $build;
  }

  /**
   * Default build in "Teaser" view mode.
   *
   * Show nodes as "cards".
   *
   * @param array $build
   *   The existing build.
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   *
   * @return array
   *   Render array.
   */
  public function buildTeaser(array $build, NodeInterface $entity) {
    // User may create a preview, so it won't have an ID or URL yet.
    $build['#url'] = !$entity->isNew() ? $entity->toUrl() : Url::fromRoute('<front>');
    $build['#title'] = $entity->label();
    $build['#body'] = $this->buildProcessedText($entity, 'body', TRUE);
    $build['#tags'] = $this->buildTags($entity);

    [$image, $image_alt] = $this->buildImage($entity, 'field_image');
    $build['#image'] = $image;
    $build['#image_alt'] = $image_alt;

    // Load Tailwind CSS framework, so our example are styled nicer.
    $build['#attached']['library'][] = 'pluggable_entity_view_builder_example/tailwind';

    $build['#theme'] = 'pluggable_entity_view_builder_example_card';

    return $build;
  }

  /**
   * Get common elements for the view modes.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   *
   * @return array
   *   Render array.
   */
  protected function getElementBase(NodeInterface $entity) {
    $element = [];

    $element['#nid'] = !$entity->isNew() ? $entity->id() : 0;
    $element['#url'] = !$entity->isNew() ? $entity->toUrl() : Url::fromRoute('<front>');
    $element['#title'] = $entity->label();

    return $element;
  }

  /**
   * Build the Hero Header section, with Title, and Background Image.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   * @param string $image_field_name
   *   Optional; The field name. Defaults to "field_image".
   *
   * @return array
   *   Render array.
   */
  protected function buildHeroHeader(NodeInterface $entity, $image_field_name = 'field_image') {
    [$image] = $this->buildImage($entity, $image_field_name);

    $element = [
      '#theme' => 'pluggable_entity_view_builder_example_content__hero_header',
      '#title' => $entity->label(),
      '#background_image' => $image,
    ];

    return $this->wrapComponentWithContainer($element, 'hero-header-wrapper', 'fluid-container-full');
  }

  /**
   * Build the content tags section.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   * @param string $field_name
   *   Optional; The term reference field name. Defaults to "field_tags".
   *
   * @return array
   *   Render array.
   */
  protected function buildContentTags(NodeInterface $entity, $field_name = 'field_tags') {
    $tags = $this->buildTags($entity, $field_name);
    if (!$tags) {
      return [];
    }

    $element = [
      '#theme' => 'pluggable_entity_view_builder_example_content__tags',
      '#tags' => $tags,
    ];

    return $this->wrapComponentWithContainer($element, 'content-tags-wrapper');
  }

  /**
   * Build a list of tags.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   * @param string $field_name
   *   Optional; The term reference field name. Defaults to "field_tags".
   *
   * @return array
   *   Render array.
   */
  protected function buildTags(NodeInterface $entity, $field_name = 'field_tags') {
    if (empty($entity->{$field_name}) || $entity->{$field_name}->isEmpty()) {
      // No terms referenced.
      return [];
    }

    $tags = [];
    foreach ($entity->{$field_name}->referencedEntities() as $term) {
      $tags[] = $this->buildTag($term);
    }

    return $tags;
  }

  /**
   * Build an image referenced in the given entity's given field name.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   * @param string $field_name
   *   Optional; The field name. Defaults to "field_image".
   *
   * @return array
   *   An array containing url and alt.
   */
  protected function buildImage(NodeInterface $entity, $field_name = 'field_image') {
    if (empty($entity->{$field_name}) || $entity->get($field_name)->isEmpty()) {
      // No field, or it's empty.
      return [NULL, NULL];
    }
    $url = $this->entityTypeManager
      ->getStorage('image_style')
      ->load(self::IMAGE_STYLE_LARGE)
      ->buildUrl($entity->get($field_name)[0]->entity->getFileUri());

    $alt = $entity->get($field_name)[0]->alt ?: '';
    return [$url, $alt];
  }

  /**
   * Build content header.
   *
   * @param \Drupal\node\NodeInterface $entity
   *   The entity.
   *
   * @return array
   *   Render array.
   */
  protected function buildContentHeader(NodeInterface $entity) {
    $element = [
      '#theme' => 'pluggable_entity_view_builder_example_content__header',
      '#title' => $entity->label(),
    ];

    return $this->wrapComponentWithContainer($element, 'content-header-wrapper', 'fluid-container-wide');
  }

}
